@@ -41,6 +41,8 @@ class AsyncSocket(socket.Socket):
             await self.server._trigger_event('message', self.sid, pkt.data)
         elif pkt.packet_type == packet.UPGRADE:
             await self.send(packet.Packet(packet.NOOP))
+        elif pkt.packet_type == packet.CLOSE:
+            await self.close(wait=False, abort=True)
         else:
             raise ValueError
 
@@ -158,7 +160,7 @@ class AsyncSocket(socket.Socket):
                         await ws.send(pkt.encode(always_bytes=False))
                 except:
                     break
-        asyncio.ensure_future(writer())
+        writer_task = asyncio.ensure_future(writer())
 
         self.server.logger.info(
             '%s: Upgrade to websocket successful', self.sid)
@@ -180,5 +182,6 @@ class AsyncSocket(socket.Socket):
             except ValueError:
                 pass
 
-        await self.close(wait=True, abort=True)
         await self.queue.put(None)  # unlock the writer task so it can exit
+        await asyncio.wait_for(writer_task, timeout=None)
+        await self.close(wait=True, abort=True)
