@@ -101,7 +101,7 @@ class Server(object):
         for mode in modes:
             try:
                 self._async = importlib.import_module(
-                    'engineio.async_' + mode).async
+                    'engineio.async_' + mode)._async
                 asyncio_based = self._async['asyncio'] \
                     if 'asyncio' in self._async else False
                 if asyncio_based != self.is_asyncio_based():
@@ -352,7 +352,11 @@ class Server(object):
             return self._unauthorized()
 
         if transport == 'websocket':
-            return s.handle_get_request(environ, start_response)
+            ret = s.handle_get_request(environ, start_response)
+            if s.closed:
+                # websocket connection ended, so we are done
+                del self.sockets[sid]
+            return ret
         else:
             s.connected = True
             headers = None
